#ifndef __SEMEM_IOCTL_H__
#define __SEMEM_IOCTL_H__	1

#include <linux/ioctl.h>
#include "uapi/semem/semem_com.h"

#define SEMEM_MODULE	'S'

#define SEMEM_IOCTL_RULE_CMD		0x60
#define SEMEM_IOCTL_LOG_CMD			0x61
#define SEMEM_IOCTL_REG_CMD			0x62
#define SEMEM_IOCTL_MEM_CMD			0x63
#define SEMEM_IOCTL_DBG_CMD			0x64

#define DIMM_ADDR_SOCKET_SHIFT		0xc
#define DIMM_ADDR_CH_SHIFT			0x8
#define DIMM_ADDR_DIMM_SHIFT		0x0


#define DIMM_ADDR(__socket, __ch, __dimm)\
	({\
		int __dimm_addr = 0;\
		__dimm_addr = (__socket << DIMM_ADDR_SOCKET_SHIFT) |\
					(__ch << DIMM_ADDR_CH_SHIFT) |\
					(__dimm << DIMM_ADDR_DIMM_SHIFT);\
		__dimm_addr;\
	})

#define DIMM_ADDR_REVERSE(__dimm_addr, __socket, __ch, __dimm)\
	({\
		__socket = (__dimm_addr >> DIMM_ADDR_SOCKET_SHIFT) & 0xf;\
		__ch = (__dimm_addr >> DIMM_ADDR_CH_SHIFT) & 0xf;\
		__dimm = (__dimm_addr >> DIMM_ADDR_DIMM_SHIFT) & 0xff;\
	})

struct semem_padding{
	char x[0];
} __attribute__((aligned(64)));
#define PACKET_PADDING(name)	struct semem_padding name

enum ioctl_rule_com_ops{
	SEMEM_RULE_SET,
	SEMEM_RULE_DISABLE,
	SEMEM_RULE_ENABLE,
	SEMEM_RULE_REMOVE,
	SEMEM_RULE_ATTR_CHG,
	SEMEM_RULE_ATTACH,
	SEMEM_RULE_DETACH
};

struct semem_ioctl_rule_com_ops_struct{
	enum ioctl_rule_com_ops rule_op;
	struct semem_range *rule;
	int ioctl_ret;
};

struct semem_ioctl_rule_replace_struct{
	struct semem_range *rule;
	struct semem_range *replaced_rule;
	int ioctl_ret;
};

struct semem_ioctl_rule_replaceable_struct{
	struct semem_range *range;
	struct semem_range *replaceable;
	unsigned int *replaceable_nr;
	int ioctl_ret;
};

#define SEMEM_IOCTL_RULE_COM_OPS \
	_IOW(SEMEM_MODULE,\
		SEMEM_IOCTL_RULE_CMD,\
		struct semem_ioctl_rule_com_ops_struct)

#define SEMEM_IOCTL_RULE_REPLACE \
	_IOWR(SEMEM_MODULE,\
		SEMEM_IOCTL_RULE_CMD,\
		struct semem_ioctl_rule_replace_struct)

#define SEMEM_IOCTL_RULE_REPLACEABLE \
	_IOR(SEMEM_MODULE,\
		SEMEM_IOCTL_RULE_CMD,\
		struct semem_ioctl_rule_replaceable_struct)

struct semem_ioctl_rule_stat_struct{
	unsigned long status;
	unsigned long status_nr;
};

#define SEMEM_IOCTL_PHYS_RULE_STAT \
	_IOR(SEMEM_MODULE, \
		SEMEM_IOCTL_RULE_CMD, \
		struct semem_ioctl_rule_stat_struct)

struct semem_ioctl_log_count_struct{
	struct semem_range *rule;
	unsigned int *log_count;
	int ioctl_ret;
};

struct semem_ioctl_log_struct{
	struct semem_range *rule;
	struct log_struct *log;
	unsigned int *log_count;
	unsigned int *log_real_cnt;
	unsigned int log_limit;
	int ioctl_ret;
};

#define SEMEM_IOCTL_LOG_COUNT \
	_IOR(SEMEM_MODULE,\
		SEMEM_IOCTL_LOG_CMD,\
		struct semem_ioctl_log_count_struct)

#define SEMEM_IOCTL_LOG \
	_IOR(SEMEM_MODULE,\
		SEMEM_IOCTL_LOG_CMD,\
		struct semem_ioctl_log_struct)

enum ioctl_addr_com_ops{
	VIRT_ADDR_DECODE,
	SYSTEM_ADDR_DECODE
};

struct semem_ioctl_addr_ops_struct{
	enum ioctl_addr_com_ops addr_op;
	unsigned long input_address;
	unsigned long *output_address;
	int ioctl_ret;
};

#define SEMEM_IOCTL_ADDR_DECODE \
	_IOWR(SEMEM_MODULE,\
		SEMEM_IOCTL_MEM_CMD,\
		struct semem_ioctl_addr_ops_struct)

#define SEMEM_IOCTL_GET_KERNEL_TEXT \
		_IOW(SEMEM_MODULE, \
			SEMEM_IOCTL_DBG_CMD, \
			struct range_struct)

struct sym_addr{
	unsigned long addr;
	char sym_name[64];
};

#define SEMEM_IOCTL_GET_KERNEL_SYMBOL \
		_IOR(SEMEM_MODULE, \
			SEMEM_IOCTL_DBG_CMD, \
			struct sym_addr)

struct rw_kernel{
	unsigned long addr;
	unsigned char value;
	int op;
};

#define SEMEM_IOCTL_RW_KERNEL \
		_IOR(SEMEM_MODULE, \
			SEMEM_IOCTL_DBG_CMD, \
			struct rw_kernel)


#define SEMEM_IOCTL_RMAP_ADDR \
		_IOR(SEMEM_MODULE, \
			SEMEM_IOCTL_DBG_CMD, \
			struct semem_ioctl_addr_ops_struct)

#endif

